import java.math.BigInteger;
import java.util.Random;

//
public class RSA {

    // No "mundo real", o numero de bits da chave deveria ser 2048 ou superior
   static final int NUMERO_BITS = 512;

   // Para o Expoente Publico, vamos usar um valor de 65537 que e' por vezes
   // usado em implementacoes de RSA
   static final BigInteger EXPOENTE_PUBLICO = BigInteger.valueOf(65537);

   // Funcao para converter uma String para um BigInteger
   // usando um array de Bytes como formato intermedio
   public static BigInteger stringToBigInt (String plaintext) {

      byte[] arrBytes = plaintext.getBytes();

      BigInteger bi = new BigInteger(arrBytes);

      return(bi);

   }  // FIM stringToBigInt


   public static void main(String[] args) {

     System.out.println("RSA");
     System.out.println("Qual e' o plaintext a cifrar?");
     String plaintext = System.console().readLine();

     // Por simplificacao, vamos converter o texto para Maiusculas:
     plaintext = plaintext.toUpperCase();

     System.out.println("Texto convertido para Maiusculas = '" + plaintext + "'");

     // Passo 1 - Vamos gerar 2 numeros primos ("p" e "q")
     // Vamos recorrer ao metodo probablePrime da classe BigInteger
     // (para o que temos de gerar tambem um numero aleatorio)
     // Como queremos que "n" (produto de "p" e "q") tenha o tamanho de "NUMERO_BITS"
     // Vamos assumir que "p" e "q" terao metade do tamanho
     BigInteger p = BigInteger.probablePrime(NUMERO_BITS / 2, new Random());
     BigInteger q = BigInteger.probablePrime(NUMERO_BITS / 2, new Random());

     // Passo 2 - Vamos apurar "n" = produto de "p" e "q"
     BigInteger n = p.multiply(q);

     // Passo 3 - Vamos calcular phi_de_n = (p -1) x (q -1)
     BigInteger p_menos_1 = p.subtract(BigInteger.ONE);
     BigInteger q_menos_1 = q.subtract(BigInteger.ONE);

     BigInteger phi_de_n = p_menos_1.multiply(q_menos_1);


     // Apenas para efeitos de ilustracao, vamos mostrar os valores apurados
     System.out.println("p = " + p);
     System.out.println("q = " + q);
     System.out.println("n = " + n);
     System.out.println("p - 1 = " + p_menos_1);
     System.out.println("q - 1 = " + q_menos_1);
     System.out.println("phi de n = " + phi_de_n);

     // Passo 4 - Vamos apurar um valor de "e" (Expoente de Chave Publica)
     // Por simplificacao, vamos usar um valor usual de 65537 para "e"
     BigInteger e = EXPOENTE_PUBLICO;
     System.out.println("e = " + e);

     // Passo 5 - Vamos agora gerar "d" (expoente de chave privada)
     BigInteger d = e.modInverse(phi_de_n);
     System.out.println("d = " + d);

     // Convertemos o plaintext (em formato de String) para um biginteger
     // (passando por um formato intermedio de um Array de Bytes)
     // A funcao "stringToBigInt" foi definida mais acima
     BigInteger biPlaintext = stringToBigInt(plaintext);
     System.out.println("Plaintext em formato de BigInteger = " + biPlaintext.toString());

     // Passo 6 - Vamos agora gerar o CipherText
     BigInteger biCiphertext = biPlaintext.modPow(e, n);
     System.out.println("Ciphertext em formato de BigInteger = " + biCiphertext.toString());


     // Abaixo seguem os passos para decifrar um CipherText

     BigInteger biPlaintextDecifrado = biCiphertext.modPow(d, n);

     // Como o plaintext tinha sido codificado num array de bytes
     // temos agora tambem de o converter num array de bytes:

     byte[] arrBytesDecifrado = biPlaintextDecifrado.toByteArray();

     // ... e agora sim vamos converte-lo na string que sera igual a string original:
     String strPlaintextDecifrado = new String(arrBytesDecifrado);

     System.out.println("Ciphertext decifrado em formato de String = " + strPlaintextDecifrado);

   }  // FIM main

} // FIM classe
